{
  This file was automatically generated on 2012-09-28.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletIO4;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_IO4_FUNCTION_SET_VALUE = 1;
  BRICKLET_IO4_FUNCTION_GET_VALUE = 2;
  BRICKLET_IO4_FUNCTION_SET_CONFIGURATION = 3;
  BRICKLET_IO4_FUNCTION_GET_CONFIGURATION = 4;
  BRICKLET_IO4_FUNCTION_SET_DEBOUNCE_PERIOD = 5;
  BRICKLET_IO4_FUNCTION_GET_DEBOUNCE_PERIOD = 6;
  BRICKLET_IO4_FUNCTION_SET_INTERRUPT = 7;
  BRICKLET_IO4_FUNCTION_GET_INTERRUPT = 8;
  BRICKLET_IO4_FUNCTION_SET_MONOFLOP = 10;
  BRICKLET_IO4_FUNCTION_GET_MONOFLOP = 11;
  BRICKLET_IO4_CALLBACK_INTERRUPT = 9;
  BRICKLET_IO4_CALLBACK_MONOFLOP_DONE = 12;

type
  TBrickletIO4NotifyInterrupt = procedure(const interruptMask: byte; const valueMask: byte) of object;
  TBrickletIO4NotifyMonoflopDone = procedure(const pinMask: byte; const valueMask: byte) of object;

  /// <summary>
  ///  Device for controlling up to 4 general purpose input/output pins
  /// </summary>
  TBrickletIO4 = class(TDevice)
  private
    interruptCallback: TBrickletIO4NotifyInterrupt;
    monoflopDoneCallback: TBrickletIO4NotifyMonoflopDone;
  protected
    procedure CallbackWrapperInterrupt(const packet: TByteArray); virtual;
    procedure CallbackWrapperMonoflopDone(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Sets the output value (high or low) with a bitmask. The bitmask
    ///  is 4 bit long, *true* refers to high and *false* refers to low.
    ///  
    ///  For example: The value 0b0011 will turn the pins 0-1 high and the
    ///  pins 2-3 low.
    ///  
    ///  <note>
    ///   This function does nothing for pins that are configured as input.
    ///   Pull-up resistors can be switched on with <see cref="BrickletIO4.TBrickletIO4.SetConfiguration"/>.
    ///  </note>
    /// </summary>
    procedure SetValue(const valueMask: byte); virtual;

    /// <summary>
    ///  Returns a bitmask of the values that are currently measured.
    ///  This function works if the pin is configured to input
    ///  as well as if it is configured to output.
    /// </summary>
    function GetValue: byte; virtual;

    /// <summary>
    ///  Configures the value and direction of the specified pins. Possible directions
    ///  are "i" and "o" for input and output.
    ///  
    ///  If the direction is configured as output, the value is either high or low
    ///  (set as *true* or *false*).
    ///  
    ///  If the direction is configured as input, the value is either pull-up or
    ///  default (set as *true* or *false*).
    ///  
    ///  For example:
    ///  
    ///  * (15, 'i', true) will set all pins of as input pull-up.
    ///  * (8, 'i', false) will set pin 3 of as input default (floating if nothing is connected).
    ///  * (3, 'o', false) will set pins 0 and 1 as output low.
    ///  * (4, 'o', true) will set pin 2 of as output high.
    /// </summary>
    procedure SetConfiguration(const pinMask: byte; const direction: char; const value: boolean); virtual;

    /// <summary>
    ///  Returns a value bitmask and a direction bitmask.
    ///  
    ///  For example: A return value of 0b0011 and 0b0101 for
    ///  direction and value means that:
    ///  
    ///  * pin 0 is configured as input pull-up,
    ///  * pin 1 is configured as input default,
    ///  * pin 2 is configured as output high
    ///  * and pin 3 is are configured as output low.
    /// </summary>
    procedure GetConfiguration(out directionMask: byte; out valueMask: byte); virtual;

    /// <summary>
    ///  Sets the debounce period of the <see cref="BrickletIO4.TBrickletIO4.Interrupt"/> callback in ms.
    ///  
    ///  For example: If you set this value to 100, you will get the interrupt
    ///  maximal every 100ms. This is necessary if something that bounces is
    ///  connected to the IO-4 Bricklet, such as a button.
    ///  
    ///  The default value is 100.
    /// </summary>
    procedure SetDebouncePeriod(const debounce: longword); virtual;

    /// <summary>
    ///  Returns the debounce period as set by <see cref="BrickletIO4.TBrickletIO4.SetDebouncePeriod"/>.
    /// </summary>
    function GetDebouncePeriod: longword; virtual;

    /// <summary>
    ///  Sets the pins on which an interrupt is activated with a bitmask.
    ///  Interrupts are triggered on changes of the voltage level of the pin,
    ///  i.e. changes from high to low and low to high.
    ///  
    ///  For example: An interrupt bitmask of 9 will enable the interrupt for
    ///  pins 0 and 3.
    ///  
    ///  The interrupt is delivered with the callback <see cref="BrickletIO4.TBrickletIO4.Interrupt"/>.
    /// </summary>
    procedure SetInterrupt(const interruptMask: byte); virtual;

    /// <summary>
    ///  Returns the interrupt bitmask as set by <see cref="BrickletIO4.TBrickletIO4.SetInterrupt"/>.
    /// </summary>
    function GetInterrupt: byte; virtual;

    /// <summary>
    ///  Configures a monoflop of the pins specified by the first parameter as 4 bit
    ///  long bitmask. The specified pins must be configured for output. Non-output
    ///  pins will be ignored.
    ///  
    ///  The second parameter is a bitmask with the desired value of the specified
    ///  output pins (*true* means high and *false* means low).
    ///  
    ///  The third parameter indicates the time (in ms) that the pins should hold
    ///  the value.
    ///  
    ///  If this function is called with the parameters ((1 &lt;&lt; 0) | (1 &lt;&lt; 3), (1 &lt;&lt; 0), 1500):
    ///  Pin 0 will get high and pin 3 will get low. In 1.5s pin 0 will get low and pin
    ///  3 will get high again.
    ///  
    ///  A monoflop can be used as a fail-safe mechanism. For example: Lets assume you
    ///  have a RS485 bus and an IO-4 Bricklet connected to one of the slave
    ///  stacks. You can now call this function every second, with a time parameter
    ///  of two seconds and pin 0 set to high. Pin 0 will be high all the time. If now
    ///  the RS485 connection is lost, then pin 0 will get low in at most two seconds.
    ///  
    ///  .. versionadded:: 1.1.1
    /// </summary>
    procedure SetMonoflop(const pinMask: byte; const valueMask: byte; const time: longword); virtual;

    /// <summary>
    ///  Returns (for the given pin) the current value and the time as set by
    ///  <see cref="BrickletIO4.TBrickletIO4.SetMonoflop"/> as well as the remaining time until the value flips.
    ///  
    ///  If the timer is not running currently, the remaining time will be returned
    ///  as 0.
    ///  
    ///  .. versionadded:: 1.1.1
    /// </summary>
    procedure GetMonoflop(const pin: byte; out value: byte; out time: longword; out timeRemaining: longword); virtual;

    /// <summary>
    ///  This callback is triggered whenever a change of the voltage level is detected
    ///  on pins where the interrupt was activated with <see cref="BrickletIO4.TBrickletIO4.SetInterrupt"/>.
    ///  
    ///  The values are a bitmask that specifies which interrupts occurred
    ///  and the current value bitmask.
    ///  
    ///  For example:
    ///  
    ///  * (1, 1) means that an interrupt on pin 0 occurred and
    ///    currently pin 0 is high and pins 1-3 are low.
    ///  * (9, 14) means that interrupts on pins 0 and 3
    ///    occurred and currently pin 0 is low and pins 1-3 are high.
    /// </summary>
    property OnInterrupt: TBrickletIO4NotifyInterrupt read interruptCallback write interruptCallback;

    /// <summary>
    ///  This callback is triggered whenever a monoflop timer reaches 0. The
    ///  parameters contain the involved pins and the current value of the pins
    ///  (the value after the monoflop).
    ///  
    ///  .. versionadded:: 1.1.1
    /// </summary>
    property OnMonoflopDone: TBrickletIO4NotifyMonoflopDone read monoflopDoneCallback write monoflopDoneCallback;
  end;

implementation

constructor TBrickletIO4.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'IO-4 Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 1;
  callbackWrappers[BRICKLET_IO4_CALLBACK_INTERRUPT] := {$ifdef FPC}@{$endif}CallbackWrapperInterrupt;
  callbackWrappers[BRICKLET_IO4_CALLBACK_MONOFLOP_DONE] := {$ifdef FPC}@{$endif}CallbackWrapperMonoflopDone;
end;

procedure TBrickletIO4.SetValue(const valueMask: byte);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_SET_VALUE, 5);
  LEConvertUInt8To(valueMask, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletIO4.GetValue: byte;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_GET_VALUE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_IO4_FUNCTION_GET_VALUE, response);
  result := LEConvertUInt8From(4, response);
end;

procedure TBrickletIO4.SetConfiguration(const pinMask: byte; const direction: char; const value: boolean);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_SET_CONFIGURATION, 7);
  LEConvertUInt8To(pinMask, 4, request);
  LEConvertCharTo(direction, 5, request);
  LEConvertBooleanTo(value, 6, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletIO4.GetConfiguration(out directionMask: byte; out valueMask: byte);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_GET_CONFIGURATION, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_IO4_FUNCTION_GET_CONFIGURATION, response);
  directionMask := LEConvertUInt8From(4, response);
  valueMask := LEConvertUInt8From(5, response);
end;

procedure TBrickletIO4.SetDebouncePeriod(const debounce: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_SET_DEBOUNCE_PERIOD, 8);
  LEConvertUInt32To(debounce, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletIO4.GetDebouncePeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_GET_DEBOUNCE_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_IO4_FUNCTION_GET_DEBOUNCE_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletIO4.SetInterrupt(const interruptMask: byte);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_SET_INTERRUPT, 5);
  LEConvertUInt8To(interruptMask, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletIO4.GetInterrupt: byte;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_GET_INTERRUPT, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_IO4_FUNCTION_GET_INTERRUPT, response);
  result := LEConvertUInt8From(4, response);
end;

procedure TBrickletIO4.SetMonoflop(const pinMask: byte; const valueMask: byte; const time: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_SET_MONOFLOP, 10);
  LEConvertUInt8To(pinMask, 4, request);
  LEConvertUInt8To(valueMask, 5, request);
  LEConvertUInt32To(time, 6, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletIO4.GetMonoflop(const pin: byte; out value: byte; out time: longword; out timeRemaining: longword);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_IO4_FUNCTION_GET_MONOFLOP, 5);
  LEConvertUInt8To(pin, 4, request);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_IO4_FUNCTION_GET_MONOFLOP, response);
  value := LEConvertUInt8From(4, response);
  time := LEConvertUInt32From(5, response);
  timeRemaining := LEConvertUInt32From(9, response);
end;

procedure TBrickletIO4.CallbackWrapperInterrupt(const packet: TByteArray);
var interruptMask: byte; valueMask: byte;
begin
  if (Assigned(interruptCallback)) then begin
    interruptMask := LEConvertUInt8From(4, packet);
    valueMask := LEConvertUInt8From(5, packet);
    interruptCallback(interruptMask, valueMask);
  end;
end;

procedure TBrickletIO4.CallbackWrapperMonoflopDone(const packet: TByteArray);
var pinMask: byte; valueMask: byte;
begin
  if (Assigned(monoflopDoneCallback)) then begin
    pinMask := LEConvertUInt8From(4, packet);
    valueMask := LEConvertUInt8From(5, packet);
    monoflopDoneCallback(pinMask, valueMask);
  end;
end;

end.
