{
  This file was automatically generated on 2012-09-28.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletVoltage;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE = 1;
  BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE = 2;
  BRICKLET_VOLTAGE_FUNCTION_SET_VOLTAGE_CALLBACK_PERIOD = 3;
  BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE_CALLBACK_PERIOD = 4;
  BRICKLET_VOLTAGE_FUNCTION_SET_ANALOG_VALUE_CALLBACK_PERIOD = 5;
  BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE_CALLBACK_PERIOD = 6;
  BRICKLET_VOLTAGE_FUNCTION_SET_VOLTAGE_CALLBACK_THRESHOLD = 7;
  BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE_CALLBACK_THRESHOLD = 8;
  BRICKLET_VOLTAGE_FUNCTION_SET_ANALOG_VALUE_CALLBACK_THRESHOLD = 9;
  BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE_CALLBACK_THRESHOLD = 10;
  BRICKLET_VOLTAGE_FUNCTION_SET_DEBOUNCE_PERIOD = 11;
  BRICKLET_VOLTAGE_FUNCTION_GET_DEBOUNCE_PERIOD = 12;
  BRICKLET_VOLTAGE_CALLBACK_VOLTAGE = 13;
  BRICKLET_VOLTAGE_CALLBACK_ANALOG_VALUE = 14;
  BRICKLET_VOLTAGE_CALLBACK_VOLTAGE_REACHED = 15;
  BRICKLET_VOLTAGE_CALLBACK_ANALOG_VALUE_REACHED = 16;

type
  TBrickletVoltageNotifyVoltage = procedure(const voltage: word) of object;
  TBrickletVoltageNotifyAnalogValue = procedure(const value: word) of object;
  TBrickletVoltageNotifyVoltageReached = procedure(const voltage: word) of object;
  TBrickletVoltageNotifyAnalogValueReached = procedure(const value: word) of object;

  /// <summary>
  ///  Device for sensing Voltages between 0 and 50V
  /// </summary>
  TBrickletVoltage = class(TDevice)
  private
    voltageCallback: TBrickletVoltageNotifyVoltage;
    analogValueCallback: TBrickletVoltageNotifyAnalogValue;
    voltageReachedCallback: TBrickletVoltageNotifyVoltageReached;
    analogValueReachedCallback: TBrickletVoltageNotifyAnalogValueReached;
  protected
    procedure CallbackWrapperVoltage(const packet: TByteArray); virtual;
    procedure CallbackWrapperAnalogValue(const packet: TByteArray); virtual;
    procedure CallbackWrapperVoltageReached(const packet: TByteArray); virtual;
    procedure CallbackWrapperAnalogValueReached(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Returns the voltage of the sensor. The value is in mV and
    ///  between 0mV and 50000mV.
    ///  
    ///  If you want to get the voltage periodically, it is recommended to use the
    ///  callback <see cref="BrickletVoltage.TBrickletVoltage.Voltage"/> and set the period with 
    ///  <see cref="BrickletVoltage.TBrickletVoltage.SetVoltageCallbackPeriod"/>.
    /// </summary>
    function GetVoltage: word; virtual;

    /// <summary>
    ///  Returns the value as read by a 12-bit analog-to-digital converter.
    ///  The value is between 0 and 4095.
    ///  
    ///  <note>
    ///   The value returned by <see cref="BrickletVoltage.TBrickletVoltage.GetVoltage"/> is averaged over several samples
    ///   to yield less noise, while <see cref="BrickletVoltage.TBrickletVoltage.GetAnalogValue"/> gives back raw
    ///   unfiltered analog values. The only reason to use <see cref="BrickletVoltage.TBrickletVoltage.GetAnalogValue"/> is,
    ///   if you need the full resolution of the analog-to-digital converter.
    ///  </note>
    ///  
    ///  If you want the analog value periodically, it is recommended to use the 
    ///  callback <see cref="BrickletVoltage.TBrickletVoltage.AnalogValue"/> and set the period with 
    ///  <see cref="BrickletVoltage.TBrickletVoltage.SetAnalogValueCallbackPeriod"/>.
    /// </summary>
    function GetAnalogValue: word; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletVoltage.TBrickletVoltage.Voltage"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletVoltage.TBrickletVoltage.Voltage"/> is only triggered if the voltage has changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetVoltageCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletVoltage.TBrickletVoltage.SetVoltageCallbackPeriod"/>.
    /// </summary>
    function GetVoltageCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the period in ms with which the <see cref="BrickletVoltage.TBrickletVoltage.AnalogValue"/> callback is triggered
    ///  periodically. A value of 0 turns the callback off.
    ///  
    ///  <see cref="BrickletVoltage.TBrickletVoltage.AnalogValue"/> is only triggered if the analog value has changed since the
    ///  last triggering.
    ///  
    ///  The default value is 0.
    /// </summary>
    procedure SetAnalogValueCallbackPeriod(const period: longword); virtual;

    /// <summary>
    ///  Returns the period as set by <see cref="BrickletVoltage.TBrickletVoltage.SetAnalogValueCallbackPeriod"/>.
    /// </summary>
    function GetAnalogValueCallbackPeriod: longword; virtual;

    /// <summary>
    ///  Sets the thresholds for the <see cref="BrickletVoltage.TBrickletVoltage.VoltageReached"/> callback. 
    ///  
    ///  The following options are possible:
    ///  
    ///  <code>
    ///   "Option", "Description"
    ///  
    ///   "'x'",    "Callback is turned off"
    ///   "'o'",    "Callback is triggered when the voltage is *outside* the min and max values"
    ///   "'i'",    "Callback is triggered when the voltage is *inside* the min and max values"
    ///   "'&lt;'",    "Callback is triggered when the voltage is smaller than the min value (max is ignored)"
    ///   "'&gt;'",    "Callback is triggered when the voltage is greater than the min value (max is ignored)"
    ///  </code>
    ///  
    ///  The default value is ('x', 0, 0).
    /// </summary>
    procedure SetVoltageCallbackThreshold(const option: char; const min: smallint; const max: smallint); virtual;

    /// <summary>
    ///  Returns the threshold as set by <see cref="BrickletVoltage.TBrickletVoltage.SetVoltageCallbackThreshold"/>.
    /// </summary>
    procedure GetVoltageCallbackThreshold(out option: char; out min: smallint; out max: smallint); virtual;

    /// <summary>
    ///  Sets the thresholds for the <see cref="BrickletVoltage.TBrickletVoltage.AnalogValueReached"/> callback. 
    ///  
    ///  The following options are possible:
    ///  
    ///  <code>
    ///   "Option", "Description"
    ///  
    ///   "'x'",    "Callback is turned off"
    ///   "'o'",    "Callback is triggered when the analog value is *outside* the min and max values"
    ///   "'i'",    "Callback is triggered when the analog value is *inside* the min and max values"
    ///   "'&lt;'",    "Callback is triggered when the analog value is smaller than the min value (max is ignored)"
    ///   "'&gt;'",    "Callback is triggered when the analog value is greater than the min value (max is ignored)"
    ///  </code>
    ///  
    ///  The default value is ('x', 0, 0).
    /// </summary>
    procedure SetAnalogValueCallbackThreshold(const option: char; const min: word; const max: word); virtual;

    /// <summary>
    ///  Returns the threshold as set by <see cref="BrickletVoltage.TBrickletVoltage.SetAnalogValueCallbackThreshold"/>.
    /// </summary>
    procedure GetAnalogValueCallbackThreshold(out option: char; out min: word; out max: word); virtual;

    /// <summary>
    ///  Sets the period in ms with which the threshold callbacks
    ///  
    ///   <see cref="BrickletVoltage.TBrickletVoltage.VoltageReached"/>, <see cref="BrickletVoltage.TBrickletVoltage.AnalogValueReached"/>
    ///  
    ///  are triggered, if the thresholds
    ///  
    ///   <see cref="BrickletVoltage.TBrickletVoltage.SetVoltageCallbackThreshold"/>, <see cref="BrickletVoltage.TBrickletVoltage.SetAnalogValueCallbackThreshold"/>
    ///  
    ///  keep being reached.
    ///  
    ///  The default value is 100.
    /// </summary>
    procedure SetDebouncePeriod(const debounce: longword); virtual;

    /// <summary>
    ///  Returns the debounce period as set by <see cref="BrickletVoltage.TBrickletVoltage.SetDebouncePeriod"/>.
    /// </summary>
    function GetDebouncePeriod: longword; virtual;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletVoltage.TBrickletVoltage.SetVoltageCallbackPeriod"/>. The parameter is the voltage of the
    ///  sensor.
    ///  
    ///  <see cref="BrickletVoltage.TBrickletVoltage.Voltage"/> is only triggered if the voltage has changed since the
    ///  last triggering.
    /// </summary>
    property OnVoltage: TBrickletVoltageNotifyVoltage read voltageCallback write voltageCallback;

    /// <summary>
    ///  This callback is triggered periodically with the period that is set by
    ///  <see cref="BrickletVoltage.TBrickletVoltage.SetAnalogValueCallbackPeriod"/>. The parameter is the analog value of the
    ///  sensor.
    ///  
    ///  <see cref="BrickletVoltage.TBrickletVoltage.AnalogValue"/> is only triggered if the voltage has changed since the
    ///  last triggering.
    /// </summary>
    property OnAnalogValue: TBrickletVoltageNotifyAnalogValue read analogValueCallback write analogValueCallback;

    /// <summary>
    ///  This callback is triggered when the threshold as set by
    ///  <see cref="BrickletVoltage.TBrickletVoltage.SetVoltageCallbackThreshold"/> is reached.
    ///  The parameter is the voltage of the sensor.
    ///  
    ///  If the threshold keeps being reached, the callback is triggered periodically
    ///  with the period as set by <see cref="BrickletVoltage.TBrickletVoltage.SetDebouncePeriod"/>.
    /// </summary>
    property OnVoltageReached: TBrickletVoltageNotifyVoltageReached read voltageReachedCallback write voltageReachedCallback;

    /// <summary>
    ///  This callback is triggered when the threshold as set by
    ///  <see cref="BrickletVoltage.TBrickletVoltage.SetAnalogValueCallbackThreshold"/> is reached.
    ///  The parameter is the analog value of the sensor.
    ///  
    ///  If the threshold keeps being reached, the callback is triggered periodically
    ///  with the period as set by <see cref="BrickletVoltage.TBrickletVoltage.SetDebouncePeriod"/>.
    /// </summary>
    property OnAnalogValueReached: TBrickletVoltageNotifyAnalogValueReached read analogValueReachedCallback write analogValueReachedCallback;
  end;

implementation

constructor TBrickletVoltage.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'Voltage Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
  callbackWrappers[BRICKLET_VOLTAGE_CALLBACK_VOLTAGE] := {$ifdef FPC}@{$endif}CallbackWrapperVoltage;
  callbackWrappers[BRICKLET_VOLTAGE_CALLBACK_ANALOG_VALUE] := {$ifdef FPC}@{$endif}CallbackWrapperAnalogValue;
  callbackWrappers[BRICKLET_VOLTAGE_CALLBACK_VOLTAGE_REACHED] := {$ifdef FPC}@{$endif}CallbackWrapperVoltageReached;
  callbackWrappers[BRICKLET_VOLTAGE_CALLBACK_ANALOG_VALUE_REACHED] := {$ifdef FPC}@{$endif}CallbackWrapperAnalogValueReached;
end;

function TBrickletVoltage.GetVoltage: word;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE, response);
  result := LEConvertUInt16From(4, response);
end;

function TBrickletVoltage.GetAnalogValue: word;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE, response);
  result := LEConvertUInt16From(4, response);
end;

procedure TBrickletVoltage.SetVoltageCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_SET_VOLTAGE_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletVoltage.GetVoltageCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletVoltage.SetAnalogValueCallbackPeriod(const period: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_SET_ANALOG_VALUE_CALLBACK_PERIOD, 8);
  LEConvertUInt32To(period, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletVoltage.GetAnalogValueCallbackPeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE_CALLBACK_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE_CALLBACK_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletVoltage.SetVoltageCallbackThreshold(const option: char; const min: smallint; const max: smallint);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_SET_VOLTAGE_CALLBACK_THRESHOLD, 9);
  LEConvertCharTo(option, 4, request);
  LEConvertInt16To(min, 5, request);
  LEConvertInt16To(max, 7, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletVoltage.GetVoltageCallbackThreshold(out option: char; out min: smallint; out max: smallint);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE_CALLBACK_THRESHOLD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_VOLTAGE_CALLBACK_THRESHOLD, response);
  option := LEConvertCharFrom(4, response);
  min := LEConvertInt16From(5, response);
  max := LEConvertInt16From(7, response);
end;

procedure TBrickletVoltage.SetAnalogValueCallbackThreshold(const option: char; const min: word; const max: word);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_SET_ANALOG_VALUE_CALLBACK_THRESHOLD, 9);
  LEConvertCharTo(option, 4, request);
  LEConvertUInt16To(min, 5, request);
  LEConvertUInt16To(max, 7, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletVoltage.GetAnalogValueCallbackThreshold(out option: char; out min: word; out max: word);
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE_CALLBACK_THRESHOLD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_ANALOG_VALUE_CALLBACK_THRESHOLD, response);
  option := LEConvertCharFrom(4, response);
  min := LEConvertUInt16From(5, response);
  max := LEConvertUInt16From(7, response);
end;

procedure TBrickletVoltage.SetDebouncePeriod(const debounce: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_SET_DEBOUNCE_PERIOD, 8);
  LEConvertUInt32To(debounce, 4, request);
  SendRequestNoResponse(request);
end;

function TBrickletVoltage.GetDebouncePeriod: longword;
var request, response: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_VOLTAGE_FUNCTION_GET_DEBOUNCE_PERIOD, 4);
  SetLength(response, 0);
  SendRequestExpectResponse(request, BRICKLET_VOLTAGE_FUNCTION_GET_DEBOUNCE_PERIOD, response);
  result := LEConvertUInt32From(4, response);
end;

procedure TBrickletVoltage.CallbackWrapperVoltage(const packet: TByteArray);
var voltage: word;
begin
  if (Assigned(voltageCallback)) then begin
    voltage := LEConvertUInt16From(4, packet);
    voltageCallback(voltage);
  end;
end;

procedure TBrickletVoltage.CallbackWrapperAnalogValue(const packet: TByteArray);
var value: word;
begin
  if (Assigned(analogValueCallback)) then begin
    value := LEConvertUInt16From(4, packet);
    analogValueCallback(value);
  end;
end;

procedure TBrickletVoltage.CallbackWrapperVoltageReached(const packet: TByteArray);
var voltage: word;
begin
  if (Assigned(voltageReachedCallback)) then begin
    voltage := LEConvertUInt16From(4, packet);
    voltageReachedCallback(voltage);
  end;
end;

procedure TBrickletVoltage.CallbackWrapperAnalogValueReached(const packet: TByteArray);
var value: word;
begin
  if (Assigned(analogValueReachedCallback)) then begin
    value := LEConvertUInt16From(4, packet);
    analogValueReachedCallback(value);
  end;
end;

end.
